#ifndef MY_TWI_H
#define MY_TWI_H
#include <mega128.h>
#include <mega128_bits.h>

/***************************************************************************************
    7..3  TWSR (  TWI).   -
             -
 ,      ""    ACK. 
         TWI   ,
      ,     
       (    , ).
  
         -
   AVR ATMega.  
****************************************************************************************/

//General TWI Master status codes:
                      
#define START                  0x08  //START has been transmitted  
#define REP_START              0x10  //Repeated START has been transmitted

//TWI Master Transmitter status codes:
                      
#define MTX_ADR_ACK            0x18  //SLA+W has been tramsmitted and ACK received
#define MTX_ADR_NACK           0x20  //SLA+W has been tramsmitted and NACK received 
#define MTX_DATA_ACK           0x28  //Data byte has been tramsmitted and ACK received
#define MTX_DATA_NACK          0x30  //Data byte has been tramsmitted and NACK received
#define MTX_ARB_LOST           0x38  //Arbitration lost in SLA+W or data bytes   

//TWI Master Receiver status codes:

#define MRX_ARB_LOST           0x38  //Arbitration lost in SLA+R or NACK bit  
#define MRX_ADR_ACK            0x40  //SLA+R has been tramsmitted and ACK received
#define MRX_ADR_NACK           0x48  //SLA+R has been tramsmitted and NACK received
#define MRX_DATA_ACK           0x50  //Data byte has been received and ACK tramsmitted
#define MRX_DATA_NACK          0x58  //Data byte has been received and NACK tramsmitted

//TWI Slave Transmitter status codes:

#define STX_ADR_ACK            0xA8  //Own SLA+R has been received; ACK has been returned
#define STX_ADR_ACK_M_ARB_LOST 0xB0  //Arbitration lost in SLA+R/W as Master; own SLA+R has been received; ACK has been returned
#define STX_DATA_ACK           0xB8  //Data byte in TWDR has been transmitted; ACK has been received
#define STX_DATA_NACK          0xC0  //Data byte in TWDR has been transmitted; NOT ACK has been received
#define STX_DATA_ACK_LAST_BYTE 0xC8  //Last data byte in TWDR has been transmitted (TWEA = 0); ACK has been received

//TWI Slave Receiver status codes:

#define SRX_ADR_ACK            0x60  //Own SLA+W has been received ACK has been returned
#define SRX_ADR_ACK_M_ARB_LOST 0x68  //Arbitration lost in SLA+R/W as Master; own SLA+W has been received; ACK has been returned
#define SRX_GEN_ACK            0x70  //General call address has been received; ACK has been returned
#define SRX_GEN_ACK_M_ARB_LOST 0x78  //Arbitration lost in SLA+R/W as Master; General call address has been received; ACK has been returned
#define SRX_ADR_DATA_ACK       0x80  //Previously addressed with own SLA+W; data has been received; ACK has been returned
#define SRX_ADR_DATA_NACK      0x88  //Previously addressed with own SLA+W; data has been received; NOT ACK has been returned
#define SRX_GEN_DATA_ACK       0x90  //Previously addressed with general call; data has been received; ACK has been returned
#define SRX_GEN_DATA_NACK      0x98  //Previously addressed with general call; data has been received; NOT ACK has been returned
#define SRX_STOP_RESTART       0xA0  //A STOP condition or repeated START condition has been received while still addressed as Slave


#define TRUE                   0x01
#define FALSE                  0x00

#define ACK                    0xD0
#define NACK                   0xD1

#define SLA_ADRESS_WR          0xD2
#define SLA_ADRESS_RD          0xD3
#define DATA                   0xD4

#define USER_ERROR             0xFE
#define TWI_ERROR              0xFF

                                   
unsigned char TWI_Start(void); 
void TWI_Stop(void);                                                     
unsigned char TWI_Status(char status);
unsigned char TWI_Master_WriteByte(unsigned char data_byte, unsigned char mode);
//unsigned char TWI_Master_ReadByte(unsigned char ACK_Status);


/*****************************************************************************
*
*     : TWI_Start
*
*    : .
*
*     :   TRUE   "" 
*                              FALSE   .
*
*
*    :    "",    
*                I2C(TWI). 
*            
*            
*****************************************************************************/ 

unsigned char TWI_Start(void)
{
  TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);    //  TWINT (  . 1,      TWINT  0) -   TWI- ,
                                             // () -   TWI.  TWEN( TWI)   1  ,     .
                                             // TWSTA    "".
  while(!(TWCR & (1<<TWINT)));               //   TWINT  1 -      "".
  
  if(TWI_Status(START)||TWI_Status(REP_START) == TRUE) //   TWI(TWSR)     
     return TRUE;                                      //  "",  TRUE.
      else
       return FALSE;                                   // (  ) - FALSE.
} 



/*****************************************************************************
*
*     : TWI_Stop
*
*    : .
*
*     : .                          
*
*    :    "",   
*                I2C(TWI). 
*            
*            
*****************************************************************************/ 

void TWI_Stop(void)			       
{                                         // ,  TWINT    -   ,  
  TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO); // TWSTO -   "" (TWEN   ,   ,  
}                                         //     ,  TWI     -    ). 



/*****************************************************************************
*
*     : TWI_Status
*
*
*    : Status -        TWSR.
*
*
*     :   TRUE    
*                           (Status)     
*                            TWI - TWSR.  - FALSE. 
*
*    :         
*             TWI - TWSR.      () 
*                .    
*                 .   TWI . . Atmel.
*
*
*           
*****************************************************************************/ 

unsigned char TWI_Status(char status)
{
  if((TWSR & 0xF8) == status) //      TWSR       TWI,   
     return TRUE;             //    .
      else                    // TRUE    (.   /  ) 
       return FALSE;          // FALSE -  .
}



/*****************************************************************************
*
*     : TWI_Master_WriteByte
*
*   
*    : data_byte -   . mode -    
*                 .     (.
*                  ): SLA_ADRESS_WR -   
*                () - +  R/W = 0 (). 
*               SLA_ADRESS_RD -       R/W = 1.
*               DATA -  . 
*
*     :   ACK -    
*                            ( )  -  ACK.
*                           NACK -  ACK . TWI_ERROR -   
*                                 TWI.
*                            USER_ERROR -    mode 
*                           .                              
*               
*    :        ()  
*             -.
*
*            
*                       
*****************************************************************************/ 

unsigned char TWI_Master_WriteByte(unsigned char data_byte, unsigned char mode)
{
  TWDR = data_byte;               //   TWDR (   ,   )  .
  TWCR = (1<<TWINT) | (1<<TWEN);  // TWINT,    TWDR       ( ).
  while(!(TWCR & (1<<TWINT)));    //      -   TWINT  1. 
                                  //        (TWSR)        .
   switch(mode)                   //          -     .
   {
     case SLA_ADRESS_WR:                      //         +  R/W   (.. ).  
          if(TWI_Status(MTX_ADR_ACK) == TRUE) //,          TWSR     + R/W=1 
             return ACK;                      // ACK  .     ACK.
              else 
                if(TWI_Status(MTX_ADR_NACK) == TRUE) //  ,       NACK (      
                   return NACK;                      //(     ).
                    else 				             //   TWSR    ,   -    
                      return TWI_ERROR;              //(  -    ).
          break;
                       
     case SLA_ADRESS_RD:                       //  ,         R/W    ()    
          if(TWI_Status(MRX_ADR_ACK) == TRUE)  //      -    -   TWSR   
             return ACK;                       // Master Receiver ( ) -     (    ). 
              else                             
                if(TWI_Status(MRX_ADR_NACK) == TRUE)
                   return NACK;
                    else
                      return TWI_ERROR;                   
          break;
          
     case DATA:                                //       .
          if(TWI_Status(MTX_DATA_ACK) == TRUE) 
             return ACK;
              else
                if(TWI_Status(MTX_DATA_NACK) == TRUE)
                   return NACK;
                    else
                      return TWI_ERROR;
          break;
          
     default: return USER_ERROR;              //          .         
   }
}



/*****************************************************************************
*
*     : TWI_Master_ReadByte
*
*    : ACK_Status -  ACK     8 , 
*               ()   ACK.   NACK  
*               . 
*
*     :  ,       ACK
*                            NACK - TWI_ERROR.                          
*
*    :    ()     . -.
*
*             
*                        
*****************************************************************************/  
/*
unsigned char TWI_Master_ReadByte(unsigned char ACK_Status)
{ 
  if(ACK_Status == ACK)                             //   .      + R/W    (),  
     TWCR = (1<<TWINT)| (1<<TWEA) | (1<<TWEN);      //      - .        
      else                                          //   ACK       (NACK),       
        TWCR = (1<<TWINT)  | (1<<TWEN);             //  (   NACK),        -  (
                                                    // ACK     TWI_Master_ReadByte  - ).
  while(!(TWCR & (1<<TWINT)));                      //   -        ,   .
  
  if(ACK_Status == ACK)                             //        ACK  NACK.
  {
    if(TWI_Status(MRX_DATA_ACK) == TRUE);
       else
         return TWI_ERROR; 
  }
     else
     {
       if(TWI_Status(MRX_DATA_NACK) == TRUE);
          else
            return TWI_ERROR; 
     }
                                  //   ACK / NACK,  TWDR       .  
  return TWDR;                    //(TWDR        ).
}
*/

#endif

